@@ -1,6 +1,7 @@
 #!/usr/bin/env python3
 
 import argparse
+import asyncio
 from datetime import datetime, timedelta
 import copy
 import io
@@ -12,6 +13,7 @@ import uuid
 from multiprocessing.pool import ThreadPool as Pool
 
 import pkg_resources
+from asyncio import AbstractEventLoop
 from c8connector import C8Connector, Sample, ConfigAttributeType, Schema
 from c8connector import ConfigProperty
 from tempfile import mkstemp
@@ -19,6 +21,7 @@ from fastavro import writer, parse_schema
 from jsonschema import Draft7Validator, FormatChecker
 from pathlib import Path
 from singer import get_logger
+from threading import Thread
 from typing import Dict
 
 from macrometa_target_bigquery import stream_utils
@@ -31,10 +34,11 @@ from macrometa_target_bigquery.exceptions import (
 LOGGER = get_logger('macrometa_target_bigquery')
 logging.getLogger('bigquery.connector').setLevel(logging.WARNING)
 
-DEFAULT_BATCH_SIZE_ROWS = 100000
+DEFAULT_BATCH_SIZE_ROWS = 10000
 DEFAULT_PARALLELISM = 0  # 0 The number of threads used to flush tables
 DEFAULT_MAX_PARALLELISM = 16  # Don't use more than this number of threads by default when flushing streams in parallel
 DEFAULT_HARD_DELETE = False
+DEFAULT_BATCH_AWAIT_TIME = 60
 
 
 class BigQueryTargetConnector(C8Connector):
@@ -110,7 +114,7 @@ class BigQueryTargetConnector(C8Connector):
             ConfigProperty('batch_size_rows', 'Batch Size', ConfigAttributeType.INT, False, False,
                            description='Maximum number of rows in each batch. At the end of each batch, '
                                        'the rows in the batch are loaded into BigQuery.',
-                           default_value='100000'),
+                           default_value='10000'),
             ConfigProperty('batch_wait_limit_seconds', 'Batch wait limit (seconds)',
                            ConfigAttributeType.INT, False, False,
                            description='Maximum time to wait for batch to reach batch_size_rows.',
@@ -219,26 +223,20 @@ def persist_lines(config, lines) -> None:
     validators = {}
     records_to_load = {}
     row_count = {}
-    flush_timestamp = {}
     stream_to_sync = {}
     total_row_count = {}
     batch_size_rows = config.get('batch_size_rows', DEFAULT_BATCH_SIZE_ROWS)
     default_hard_delete = config.get('hard_delete', DEFAULT_HARD_DELETE)
     hard_delete_mapping = config.get('hard_delete_mapping', {})
-    batch_wait_limit_seconds = config.get('batch_wait_limit_seconds', None)
+    time_schedule = {
+        'batch_wait_limit_seconds': config.get('batch_wait_limit_seconds', DEFAULT_BATCH_AWAIT_TIME),
+        'last_executed_time': datetime.now(),
+    }
+    event_loop = setup_flush_task(time_schedule, records_to_load, row_count, stream_to_sync, config,
+                                  state, flushed_state)
 
     # Loop over lines from stdin
     for line in lines:
-        # Check to see if any streams should be flushed based on time.
-        # This assumes that each individual record takes a negligible period
-        # of time to be processed.
-        streams_to_flush_timestamp = set()
-        if batch_wait_limit_seconds:
-            streams_to_flush_timestamp = {
-                stream for stream, timestamp in flush_timestamp.items()
-                if datetime.utcnow() >= timestamp + timedelta(seconds=batch_wait_limit_seconds)
-            }
-
         try:
             o = json.loads(line)
         except json.decoder.JSONDecodeError:
@@ -289,25 +287,18 @@ def persist_lines(config, lines) -> None:
             else:
                 records_to_load[stream][primary_key_string] = o['record']
 
-            flush = False
             if row_count[stream] >= batch_size_rows:
-                flush = True
                 LOGGER.info("Flush triggered by batch_size_rows (%s) reached in %s",
                              batch_size_rows, stream)
-            elif streams_to_flush_timestamp:
-                flush = True
-                LOGGER.info("Flush triggered by batch_wait_limit_seconds (%s)",
-                            batch_wait_limit_seconds)
 
-            if flush:
                 # flush all streams, delete records if needed, reset counts and then emit current state
                 if config.get('flush_all_streams'):
                     filter_streams = None
                 else:
-                    filter_streams = list(streams_to_flush_timestamp | {stream})
+                    filter_streams = [stream]
 
                 # Flush and return a new state dict with new positions only for the flushed streams
-                flushed_state, flushed_timestamps = flush_streams(
+                flushed_state = flush_streams(
                     records_to_load,
                     row_count,
                     stream_to_sync,
@@ -316,10 +307,9 @@ def persist_lines(config, lines) -> None:
                     flushed_state,
                     filter_streams=filter_streams)
 
-                flush_timestamp.update(flushed_timestamps)
-
                 # emit last encountered state
                 emit_state(copy.deepcopy(flushed_state))
+                time_schedule['last_executed_time'] = datetime.now()
 
         elif t == 'SCHEMA':
             if 'stream' not in o:
@@ -337,14 +327,12 @@ def persist_lines(config, lines) -> None:
                 if config.get('flush_all_streams'):
                     filter_streams = None
                 else:
-                    filter_streams = list(streams_to_flush_timestamp | {stream})
+                    filter_streams = [stream]
 
-                flushed_state, flushed_timestamps = flush_streams(
+                flushed_state = flush_streams(
                     records_to_load, row_count, stream_to_sync, config, state, flushed_state, filter_streams=filter_streams
                 )
 
-                flush_timestamp.update(flushed_timestamps)
-
                 # emit latest encountered state
                 emit_state(flushed_state)
 
@@ -384,7 +372,6 @@ def persist_lines(config, lines) -> None:
             records_to_load[stream] = {}
             row_count[stream] = 0
             total_row_count[stream] = 0
-            flush_timestamp[stream] = datetime.utcnow()
 
         elif t == 'ACTIVATE_VERSION':
             stream = o['stream']
@@ -415,10 +402,12 @@ def persist_lines(config, lines) -> None:
     # then flush all buckets.
     if sum(row_count.values()) > 0:
         # flush all streams one last time, delete records if needed, reset counts and then emit current state
-        flushed_state, _ = flush_streams(records_to_load, row_count, stream_to_sync, config, state, flushed_state)
+        flushed_state = flush_streams(records_to_load, row_count, stream_to_sync, config, state, flushed_state)
+        time_schedule['last_executed_time'] = datetime.now()
 
     # emit latest state
     emit_state(copy.deepcopy(flushed_state))
+    event_loop.stop()
 
 
 # pylint: disable=too-many-arguments
@@ -517,9 +506,8 @@ def flush_streams(
         else:
             flushed_state = copy.deepcopy(state)
 
-    flushed_timestamps = {stream: datetime.utcnow() for stream in streams_to_flush}
     # Return with state message with flushed positions
-    return flushed_state, flushed_timestamps
+    return flushed_state
 
 
 def load_stream_batch(stream, records_to_load, row_count, db_sync, delete_rows=False):
@@ -546,6 +534,39 @@ def flush_records(stream, records_to_load, row_count, db_sync):
     os.remove(csv_file)
 
 
+def setup_flush_task(time_schedule, streams, row_count, stream_to_sync, config, state, flushed_state,
+                     filter_streams=None) -> AbstractEventLoop:
+    event_loop = asyncio.new_event_loop()
+    Thread(target=start_background_loop, args=(event_loop,), daemon=True).start()
+    asyncio.run_coroutine_threadsafe(
+        flush_task(time_schedule, streams, row_count, stream_to_sync, config, state, flushed_state, filter_streams),
+        event_loop)
+    return event_loop
+
+
+async def flush_task(time_schedule, streams, row_count, stream_to_sync, config, state, flushed_state,
+                     filter_streams=None) -> None:
+    while True:
+        timedelta = datetime.now() - time_schedule['last_executed_time']
+        if (
+            timedelta.total_seconds() >= time_schedule['batch_wait_limit_seconds']
+            and sum(row_count.values()) > 0
+        ):
+            # flush all streams one last time, delete records if needed, reset counts and then emit current state.
+            flushed_state = flush_streams(streams, row_count, stream_to_sync, config, state, flushed_state,
+                                          filter_streams)
+            # emit latest state
+            emit_state(copy.deepcopy(flushed_state))
+            time_schedule['last_executed_time'] = datetime.now()
+        # Add sleep statement to ensure periodic execution
+        await asyncio.sleep(time_schedule['batch_wait_limit_seconds'])
+
+
+def start_background_loop(loop: asyncio.AbstractEventLoop) -> None:
+    asyncio.set_event_loop(loop)
+    loop.run_forever()
+
+
 def create_credentials_file(config: Dict) -> Dict:
     path_uuid = uuid.uuid4().hex
     try:
